<?php

namespace App\Http\Controllers;

use App\Models\Buyer;
use App\Models\Seller;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Laravel\Socialite\Facades\Socialite;
use Illuminate\Support\Str;

class loginController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('login');
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $request->validate([
            'email' => 'required|email',
            'password' => 'required|string|min:8',
        ]);

        // echo "<pre>";print_r($request->all());exit;

        $seller = Seller::where('email', $request->email)->first();
        if ($seller && base64_decode($seller->mypass) === $request->password) {
            Auth::guard('seller')->login($seller);
            return redirect()->route('business.dashboard')->with('success', 'Logged in successfully as Seller.');
        }

        // Check for Buyer
        $buyer = Buyer::where('email', $request->email)->first();
        if ($buyer && base64_decode($buyer->mypass) === $request->password) {
            Auth::guard('buyer')->login($buyer);
            return redirect()->route('business.dashboard')->with('success', 'Logged in successfully as Buyer.');
        }

        if (Auth::attempt($request->only('email', 'password'))) {
            return redirect()->route('business.dashboard')->with('success', 'Logged in successfully.');
        }

        return redirect()->back()->withErrors([
            'email' => 'The provided credentials do not match our records.',
        ])->withInput();
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        //
    }

    public function logout(Request $request)
    {
        if (Auth::guard('seller')->check()) {
            Auth::guard('seller')->logout();
            $request->session()->invalidate();
            $request->session()->regenerateToken();
            return redirect()->route('login.index')->with('success', 'Logged out successfully from Seller account.');
        }

        if (Auth::guard('buyer')->check()) {
            Auth::guard('buyer')->logout();
            $request->session()->invalidate();
            $request->session()->regenerateToken();
            return redirect()->route('login.index')->with('success', 'Logged out successfully from Buyer account.');
        }

        Auth::logout();
        $request->session()->invalidate();
        $request->session()->regenerateToken();

        return redirect()->route('login.index')->with('success', 'Logged out successfully from Web account.');
    }

    /**
     * Redirect to Google for registration.
     */
    public function redirectToGoogleForRegistration(Request $request)
    {
        $userType = $request->query('type');

        if (!in_array($userType, ['seller', 'buyer'])) {
            return redirect()->back()->with('error', 'Invalid user type.');
        }

        session(['user_type' => $userType]);

        return Socialite::driver('google')
            ->scopes(['openid', 'profile', 'email'])
            ->with(['state' => $userType])
            ->redirect();
    }


    /**
     * Handle Google callback for registration.
     */
    public function handleGoogleRegistrationCallback(Request $request)
    {
        try {
            $googleUser = Socialite::driver('google')->stateless()->user();

            $userType = $request->get('state');

            if ($userType === 'seller') {
                $user = Seller::where('email', $googleUser->email)->first();

                if (!$user) {
                    $user = Seller::create([
                        'name' => $googleUser->name,
                        'email' => $googleUser->email,
                        'google_id' => $googleUser->id,
                        'password' => Hash::make(Str::random(10)),
                        'mypass' => base64_encode(Str::random(10)),
                        'role_id' => 2,
                    ]);
                }
            } elseif ($userType === 'buyer') {
                $user = Buyer::where('email', $googleUser->email)->first();

                if (!$user) {
                    $user = Buyer::create([
                        'name' => $googleUser->name,
                        'email' => $googleUser->email,
                        'google_id' => $googleUser->id,
                        'password' => Hash::make(Str::random(10)),
                        'mypass' => base64_encode(Str::random(10)),
                        'designation' => "CEO",
                        'country' => 13,
                        'state' => "Melbourne",
                        'preferred_industry' => ["IT & Software Company"],
                        'locations' => ["Sydney"],
                        'role_id' => 3,
                    ]);
                }
            } else {
                throw new Exception("Invalid user type");
            }

            return redirect()->route('home');

        } catch (Exception $e) {
            Log::error('Google registration failed: ' . $e->getMessage());
            return redirect()->back()->with('error', 'Failed to register using Google.');
        }
    }

    public function register(Request $request)
    {
        $userType = $request->query('type');

        if (!in_array($userType, ['seller', 'buyer'])) {
            return redirect()->back()->with('error', 'Invalid user type.');
        }

        session(['user_type' => $userType]);

        if ($userType === 'seller') {
            $request->validate([
                'name' => 'required|string|max:255',
                'email' => 'required|string|email|max:255|unique:users',
                'password' => 'required|string|min:8',
                'location' => 'required|string',
                'phone_number' => 'required|string',
            ]);

            $user = Seller::where('email', $request->email)->first();

            if (!$user) {
                $seller = new Seller();
                $seller->name = $request->input('name');
                $seller->email = $request->input('email');
                $seller->password = Hash::make($request->input('password'));
                $seller->mypass = base64_encode($request->input('password'));
                $seller->city = $request->input('location');
                $seller->phone_number = $request->input('phone_number');
                $seller->role_id = 2;
                // echo '<pre>';print_r($seller);exit;
                $seller->save();

                $user = $seller;
            }
        } elseif ($userType === 'buyer') {
            $request->validate([
                'name' => 'required|string|max:255',
                'email' => 'required|string|email|max:255|unique:users',
                'password' => 'required|string|min:8',
                'location' => 'required|string',
                'phone_number' => 'required|string',
            ]);

            $user = Buyer::where('email', $request->email)->first();

            if (!$user) {
                $buyer = new Buyer();
                $buyer->name = $request->input('name');
                $buyer->email = $request->input('email');
                $buyer->password = Hash::make($request->input('password'));
                $buyer->mypass = base64_encode($request->input('password'));
                $buyer->city = $request->input('location');
                $buyer->phone_number = $request->input('phone_number');
                $buyer->role_id = 3;
                // echo '<pre>';print_r($buyer);exit;
                $buyer->save();

                $user = $buyer;
            }
        } else {
            throw new Exception("Invalid user type");
        }

        Auth::guard($userType === 'seller' ? 'seller' : 'buyer')->login($user);

        return redirect()->route('business.dashboard')->with('success', 'Registration successful.');
    }
}
